/* include library for HC-SR04 sonar */
#include <NewPing.h>  

/* include library for IR signals */
#include "IRremote.h"

/* define notes */
#include "pitches.h"   

/* Mario main theme melody - duration of notes */
#include "melody.h"

/* define pins */
#define TRIGGER_PIN   12    // Trigger PIN 
#define ECHO_PIN      11    // Echo PIM 
#define MAX_DISTANCE 300    // define max distance for Sonar 
#define LED1          2     // green LED
#define LED2          3     // red LED 
#define BUZZER        10    // buzzer PIN
#define HEADPHONES    8     // headphone PIN
#define INDOOR_PIN     4    // state PIN 
#define NUMBER_STARTUP 3    // counter for loops in script
#define NUMBER_CHANGESTATE_INDOOR   5   // number of beeps for changing state I.
#define NUMBER_CHANGESTATE_OUTDOOR 10   // number of beeps for changing state II.
#define IRreceiver      7   // IR receiver PIN
#define ANALOG_PHOTORES 0   // photoresistor analog PIN
#define VAL_PROBE       3   // water level indicator PIN


//====================GLOBAL VARIABLES=======================
/* set indoor and outdoor disance mode */
unsigned char mode_distance[] = {40, 70}; 
unsigned char state;        // state (0-indoor, 1-outdoor)
int pass=0;                 // varibale for loop count - when having warning - put bigger delay
int photocellReading;       // light value
int moisture;               // moisture value
//====================GLOBAL VARIABLES END=======================


//====================DECLARE OBJECTS=======================
IRrecv irrecv(IRreceiver);  // create instance of 'IRrecv'

decode_results results;     // create instance of 'decode_results'

NewPing sonar(TRIGGER_PIN, ECHO_PIN, MAX_DISTANCE);   // new instance of 'sonar'
//====================DECLARE OBJECTS=======================


//====================DECLARE FUNCTIONS=======================

/* on state change play "var" number of beeps */
void play_change(int var){
  int i;
  digitalWrite(HEADPHONES, LOW);
  for(i=0;i<var;i++){
      buzz(HEADPHONES, 260, 80);
      delay(250);
  }
}


/* play N number of beeps, where N is light sensor value in hundereds */
void play_value(int var){
  int value=var/100;
  if(value==0){
    buzz(HEADPHONES, 260, 1000);
    delay(250);
  }else{
    for(int i=0;i<value;i++){
      buzz(HEADPHONES, 260, 80);
      delay(250);
    }
  }
  
}

/* read in what state is user */
int read_user_state(){  
  if( digitalRead(INDOOR_PIN) == 1 )
    return 0;     //INDOOR STATE
  else
    return 1;     //OUTDOOR STATE
}

/* take action based by IR code received and take regular actions */ 
void IRsignal(){
  Serial.println("Play me some song please !");
  switch(results.value)       //check the IR signal received 
  {
  case 0xFFA25D:{             // TURN ON button = find my stick

       Serial.println(" 'Mario Theme'");
       int size = sizeof(melody) / sizeof(int);
       for (int thisNote = 0; thisNote < size; thisNote++) {
  
         // to calculate the note duration, take one second
         // divided by the note type.
         // e.g. quarter note = 1000 / 4, eighth note = 1000/8, etc.
         int noteDuration = 1000/tempo[thisNote];
  
         buzz(BUZZER, melody[thisNote],noteDuration);
  
         // to distinguish the notes, set a minimum time between them.
         // the note's duration + 30% seems to work well:
         int pauseBetweenNotes = noteDuration * 1.30;
         delay(pauseBetweenNotes);
  
         // stop the tone playing:
         buzz(BUZZER, 0, noteDuration);
       }
   break;

    
   }
   case 0xFFE21D:{    // MENU button = what is light result
      photocellReading = analogRead(ANALOG_PHOTORES);
      Serial.print("Analog reading = ");
      Serial.println(photocellReading);
      play_value(photocellReading);     // call function for playing Light reading
      break;
   }
   
   default:       // if user pushed some other button
      Serial.println(" other button");
  }
}


/* play the tone on targetPin with arguments as frequency and period */
void buzz(int targetPin, long frequency, long length) {

 
  long delayValue = 1000000/frequency/2;        // calculate the delay value between transitions
   Serial.println(delayValue);
  //// 1 second's worth of microseconds, divided by the frequency, then split in half since
  //// there are two phases to each cycle
  long numCycles = frequency * length/ 1000;    // calculate the number of cycles for proper timing
  //// multiply frequency, which is really cycles per second, by the number of seconds to 
  //// get the total number of cycles to produce
  for (long i=0; i < numCycles; i++){     // for the calculated length of time...
      digitalWrite(targetPin,HIGH);       // write the buzzer pin high to push out the diaphram
      delayMicroseconds(delayValue);      // wait for the calculated delay value
      digitalWrite(targetPin,LOW);        // write the buzzer pin low to pull back the diaphram
      delayMicroseconds(delayValue);      // wait again or the calculated delay value
  }

}


/* main SETUP loop */
void setup() {
  pinMode(LED1, OUTPUT);
  pinMode(LED2, OUTPUT);
  pinMode(INDOOR_PIN, INPUT);
  pinMode(BUZZER, OUTPUT);
  pinMode(HEADPHONES, OUTPUT);
  state=read_user_state();      // read user state on startup
  play_change(NUMBER_STARTUP);  // play sound on startup
  irrecv.enableIRIn();          // start the IR receiver
  Serial.begin(9600);
}


/* main loop */
void loop() {
  
   moisture = analogRead(VAL_PROBE);    // read moisture

   if(moisture<1000){         //if moisture level is less than some limit (1000), turn on the buzzer for 1 second
    digitalWrite(BUZZER, HIGH);
    delay(1000);
    digitalWrite(BUZZER, LOW);
    delay(1000);
   }

   if (irrecv.decode(&results))     // check if you received an IR signal
    {
      Serial.println(results.value, HEX);   // print result code of IF
      IRsignal();                           // process IRsignal
      irrecv.resume();                      // receive next value
      delay(200); 
    }


  
  int udaljenost = sonar.ping_cm();     // get sonar output in cm

  if( state != read_user_state() ){     // check if the state has changed
    Serial.println("User has changed his state!");
    state=read_user_state();
    if(state==0)
      play_change(NUMBER_CHANGESTATE_INDOOR);   // inform user about current state, state = 0 -> 5 times on/off buzzer
    else
      play_change(NUMBER_CHANGESTATE_OUTDOOR);  // inform user about current state, state = 1 -> 10 times on/off buzzer
  }
  
  /* check if sonar is close to an object - warning */
  if ( udaljenost < mode_distance[state] && udaljenost!=0) {
    Serial.print("Warning, you're about to hit something !");
    /* set LED output */
    digitalWrite(LED1,LOW);
    digitalWrite(LED2,HIGH); 

    /* if user is very close to object (d/3), than beep faster */
    if( udaljenost <=  mode_distance[state] / 3){
        Serial.println("Big warning, you're surely gonna hit something!");
        buzz(HEADPHONES, 260, 40);
    /* else beep slower */
    }else if(pass%2){
      buzz(HEADPHONES, 260, 80);
      pass++;
    }else{
      digitalWrite(HEADPHONES, LOW);
      pass++;
    }
    
  /* if  sonar is not close to an object, set LED output */
  } else {
    digitalWrite(LED1,HIGH);
    digitalWrite(LED2,LOW);
    digitalWrite(HEADPHONES, LOW);
  }


  /* Print state, distance and moisture*/
  Serial.print("Current state:  ");
  Serial.print(state);
  Serial.print("   Current distance: ");
  Serial.print(udaljenost);
  Serial.print("cm");
  Serial.print("   Current moisture: ");
  Serial.println(moisture);

  delay(200);   //refresh every 0.2 seconds = 200 ms
  
}

